﻿using Borepin.ViewModel;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Windows.Input;
using Prism.Commands;
using Prism.Navigation;
using Borepin.Service.BFFH;
using Borepin.Base;
using FabAccessAPI.Schema;

namespace Borepin.PageModel
{
    public class MachineListPageModel : PageModelBase
    {
        #region Private Properties
        private readonly IBFFHService _BFFHService;
        #endregion

        #region Constructors
        public MachineListPageModel(INavigationService navigationService, IBFFHService bFFHService) : base(navigationService)
        {
            _BFFHService = bFFHService;

            SelectInstanceCommand = new DelegateCommand<object>(SelectInstanceCommandExecuted);
        }
        #endregion

        #region Data
        public override async Task LoadData()
        {
            if (_BFFHService.ActiveConnection == null)
            {
                return;
            }
            else
            {
                IsConnected = true;
            }

            IMachineSystem machineInterface = await _BFFHService.GetMachineSystemInterface();
            MachineSystem.IInfoInterface infoInterface = await machineInterface.Info();

            IReadOnlyList<Machine> machine_list = await infoInterface.GetMachineList();

            List<MachineListItemViewModel> viewmodel_list = new List<MachineListItemViewModel>();
            foreach (Machine machine in machine_list)
            {
                viewmodel_list.Add(new MachineListItemViewModel(machine));
            }

            MachineListItemViewModel_List = viewmodel_list;

            IsBusy = false;
        }
        #endregion

        #region Properties
        private bool _IsConnected = false;
        public bool IsConnected
        {
            get => _IsConnected;
            set => SetProperty(ref _IsConnected, value);
        }

        private List<MachineListItemViewModel> _MachineListItemViewModel_List;
        public List<MachineListItemViewModel> MachineListItemViewModel_List
        {
            get => _MachineListItemViewModel_List;
            set => SetProperty(ref _MachineListItemViewModel_List, value);
        }
        #endregion

        #region Commands
        private ICommand _SelectInstanceCommand;
        public ICommand SelectInstanceCommand
        {
            get => _SelectInstanceCommand;
            set => SetProperty(ref _SelectInstanceCommand, value);
        }
        private async void SelectInstanceCommandExecuted(object obj)
        {
            MachineListItemViewModel viewmodel = obj as MachineListItemViewModel;

            NavigationParameters parameters = new NavigationParameters
            {
                { "id", viewmodel.Instance.Id }
            };

            INavigationResult result = await _NavigationService.NavigateAsync($"MachinePage", parameters);
            if (!result.Success)
            {
                System.Diagnostics.Debugger.Break();
            }
        }
        #endregion

        #region INavigationAware
        public override void OnNavigatedFrom(INavigationParameters parameters)
        {
            
        }

        public override void OnNavigatedTo(INavigationParameters parameters)
        {
            IsBusy = true;
            Task.Run(LoadData);
        }
        #endregion
    }
}
